"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Debugger = require("debug");
const qs = require("qs");
const isBuffer = require("is-buffer");
const methods_1 = require("./methods");
const model_1 = require("./model");
exports.Model = model_1.default;
const result_1 = require("./result");
const debug = Debugger('akita:client');
const INSTANCES = {};
function resolve(key) {
    if (!INSTANCES[key]) {
        INSTANCES[key] = create();
    }
    return INSTANCES[key];
}
function create(options) {
    const client = function client(path) {
        var _a;
        return _a = class AnonymousModel extends model_1.default {
            },
            _a.path = path,
            _a.client = client,
            _a;
    };
    client._options = options || {};
    client._count = 0;
    client.create = create;
    client.resolve = resolve;
    client.setOptions = (opts) => {
        client._options = Object.assign({}, client._options, opts);
    };
    function getFormDataClass() {
        let FormData = client._options.FormData;
        if (!FormData && typeof window === 'object' && window.FormData) {
            FormData = window.FormData;
        }
        else if (!FormData && typeof global === 'object' && global.FormData) {
            FormData = global.FormData;
        }
        return FormData;
    }
    client.request = function request(path, init, query, reducer) {
        init = Object.assign({}, init);
        let queryParams = Object.assign({}, init.query);
        delete init.query;
        let queryString = qs.stringify(queryParams);
        if (queryString) {
            path += `?${queryString}`;
        }
        if (init.headers) {
            init.headers = Object.assign({}, init.headers);
        }
        if (init.body && typeof init.body === 'object') {
            let body = init.body;
            let FormData = getFormDataClass();
            if (init.method === 'UPLOAD') {
                init.method = 'POST';
                if (!FormData) {
                    throw new Error('Akita Error: Can not resolve FormData class when use upload method');
                }
                if (!(body instanceof FormData)) {
                    let form = new FormData();
                    Object.keys(body).forEach((name) => form.append(name, body[name]));
                    init.body = form;
                }
            }
            else if (!isBuffer(body) && !(FormData && body instanceof FormData) && !(body instanceof ArrayBuffer)) {
                if (!init.headers) {
                    init.headers = {};
                }
                if (!init.headers['Content-Type'] || init.headers['Content-Type'] === 'application/json') {
                    init.headers['Content-Type'] = 'application/json';
                    init.body = JSON.stringify(body);
                }
                else if (init.headers['Content-Type'] === 'application/x-www-form-urlencoded') {
                    init.body = qs.stringify(body);
                }
                else {
                    throw new Error(`Akita Error: Unsupported Content-Type ${init.headers['Content-Type']}`);
                }
            }
        }
        if (client._options.init) {
            init = Object.assign({}, client._options.init, init);
            if (client._options.init.headers) {
                init.headers = Object.assign({}, client._options.init.headers, init.headers);
            }
        }
        if (!init.headers) {
            init.headers = {};
        }
        if (!init.headers['User-Agent']) {
            init.headers['User-Agent'] = 'Akita/0.6.1 (+https://github.com/maichong/akita)';
        }
        let apiRoot = client._options.apiRoot;
        if (apiRoot) {
            if (apiRoot[apiRoot.length - 1] === '/' && path[0] === '/') {
                path = path.substring(1);
            }
            else if (apiRoot[apiRoot.length - 1] !== '/' && path[0] !== '/') {
                path = `/${path}`;
            }
            path = apiRoot + path;
        }
        if (debug.enabled) {
            debug(init.method, path, JSON.stringify(init));
        }
        let fetch = client._options.fetch;
        if (!fetch) {
            if (typeof window !== 'undefined') {
                fetch = window.fetch;
            }
            else if (typeof global !== 'undefined') {
                fetch = global.fetch;
            }
        }
        client._count += 1;
        return new result_1.default(fetch, path, init, query, reducer);
    };
    methods_1.default.forEach((method) => {
        client[method] = function (path, init) {
            init = init || {};
            init.method = method.toUpperCase();
            return client.request(path, init);
        };
    });
    return client;
}
exports.default = resolve('default');
